using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using NDesk.Options;


namespace wavutil
{
    /// <summary>
    /// Process command line arguments
    /// </summary>
    static public class Args
    {
        // Error indicators
        static public bool ErrorFlag;
        static public string ErrorString;

        //
        // Each possible option has two asociated variables: 
        // Data_X : data value
        // Flag_X : bool type that indicates wheter the option has been specified /////////////////////////////////////X
        // and its value is valid
        static public bool   Flag_IFile = false;
        static public string Data_IFile;
        static public bool   Flag_OFile = false;
        static public string Data_OFile;
        //
        static public bool   Flag_Help = false;
        static public bool   Flag_Raw2Wav = false;
        static public bool   Flag_Wav2Raw = false;
        static public bool   Flag_Resample = false;
        //
        static public bool   Flag_Fmt = false;
        static public RawFileFormat Data_Fmt;
        static public bool   Flag_Fs = false;
        static public int    Data_Fs;
        //
        // P - interpola. factor
        static public bool   Flag_P = false;
        static public byte   Data_P;
        //
        // Q - decimation factor
        static public bool   Flag_Q = false;
        static public byte   Data_Q;
        //
        // P/Q (decimal representation)
        static public bool   Flag_PQ = false;
        static public float  Data_PQ;

        //
        static public void CmdLineProc(string[] args)
        {
            // Option Set
            OptionSet p = new OptionSet()
              .Add("h|help",     delegate(string v) { if (v != null) Flag_Help     = true; })
              .Add("s|resample", delegate(string v) { if (v != null) Flag_Resample = true; })
              .Add("w|wav2raw",  delegate(string v) { if (v != null) Flag_Wav2Raw  = true; })
              .Add("r|raw2wav",  delegate(string v) { if (v != null) Flag_Raw2Wav  = true; })
              .Add("P=",         delegate(string v) { Proc_P(v);  })
              .Add("Q=",         delegate(string v) { Proc_Q(v); })
              .Add("fs=",        delegate(string v) { Proc_Fs(v);  })
              .Add("fmt=",       delegate(string v) { Proc_Fmt(v); });


            ErrorFlag = false;
            ErrorString = "No error";

            // Let's check args
            List<string> extra = p.Parse(args);

            if (extra.Count != 2)
            {
                ErrorFlag = true;
                ErrorString = "Incorrect number of mandatory parameters";
                return;
            }

            //
            Flag_IFile = true;
            Data_IFile = extra[0];
            //
            Flag_OFile = true;
            Data_OFile = extra[1];

            // Check for input and output files
            if (!File.Exists(Data_IFile))
            {
                ErrorFlag = true;
                ErrorString = "Input file " + Data_IFile + " does not exist";
                return;
            }

            if (File.Exists(Data_OFile))
            {
                ErrorFlag = true;
                ErrorString = "Output file " + Data_OFile + " already exists";
                return;
            }
        }

        static private void Proc_P(string v)
        {
            try
            {
                Data_P = Convert.ToByte(v);
                Flag_P = true;
                return;
            }
            catch (System.OverflowException)
            { ErrorString = "--P value overflows"; }
            catch (System.FormatException)
            { ErrorString = "--P value format is bad"; }
            catch (System.ArgumentException)
            { ErrorString = "--P value is null"; }

            ErrorFlag = true;
        }

        static private void Proc_Q(string v)
        {
            try
            {
                Data_Q = Convert.ToByte(v);
                Flag_Q = true;
                return;
            }
            catch (System.OverflowException)
            { ErrorString = "--Q value overflows"; }
            catch (System.FormatException)
            { ErrorString = "--Q value format is bad"; }
            catch (System.ArgumentException)
            { ErrorString = "--Q value is null"; }

            ErrorFlag = true;
        }


        //
        //static private void Proc_Rf(string v)
        //{
        //    try
        //    {
        //        Data_Rf = Convert.ToDouble(v);
        //        Flag_RF = true;
        //        return;
        //    }
        //    catch (System.OverflowException)
        //    { ErrorString = "--rf value overflows"; }
        //    catch (System.FormatException)
        //    { ErrorString = "--rf value format is bad"; }
        //    catch (System.ArgumentException)
        //    { ErrorString = "--rf value is null"; }

        //    ErrorFlag = true;
        //}


        //
        static private void Proc_Fs(string v)
        {
            try
            {
                Data_Fs = Convert.ToInt32(v);
                Flag_Fs = true;
                return;
            }
            catch (System.OverflowException)
            { ErrorString = "--fs value overflows"; }
            catch (System.FormatException)
            { ErrorString = "--fs value format is bad"; }
            catch (System.ArgumentException)
            { ErrorString = "--fs value is null"; }

            ErrorFlag = true;
        }

        static private void Proc_Fmt(string v)
        {
            string[] fmtstr = new string[] { "U08", "S08", "U16", "S16", "U24", "S24", "F32", "F64" };

            v = v.ToUpper();

            /**/ if (v == fmtstr[0]) Data_Fmt = RawFileFormat.U_PCM08;
            else if (v == fmtstr[1]) Data_Fmt = RawFileFormat.S_PCM08;
            else if (v == fmtstr[2]) Data_Fmt = RawFileFormat.U_PCM16;
            else if (v == fmtstr[3]) Data_Fmt = RawFileFormat.S_PCM16;
            else if (v == fmtstr[4]) Data_Fmt = RawFileFormat.U_PCM24;
            else if (v == fmtstr[5]) Data_Fmt = RawFileFormat.S_PCM24;
            else if (v == fmtstr[6]) Data_Fmt = RawFileFormat.FLOAT32;
            else if (v == fmtstr[7]) Data_Fmt = RawFileFormat.FLOAT64;
            else ErrorFlag = true;

            if (!ErrorFlag)
            {
                Flag_Fmt = true;
            }
            else
            {
                ErrorFlag = true;
                ErrorString = "Fmt format is bad";
            }
        }
    }
}
