using System;
using System.Collections.Generic;
using System.Text;

namespace ima2wav
{
    /// <summary>
    /// Codec IMA-ADPCM (Decoder) 
    /// </summary>
    public class ImaAdpcmDecoder
    {
        // Internal state
        sbyte _index; // index for step table          __state
        int   _pred;  // predicted signal (acumulated) __state

        sbyte[] _indexTable = {-1, -1, -1, -1, 2, 4, 6, 8, -1, -1, -1, -1, 2, 4, 6, 8};
        int[]   _deltaTable = { 7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 21, 23, 25, 28, 
            31, 34, 37, 41, 45, 50, 55, 60, 66, 73, 80, 88, 97, 107, 118, 130, 143, 157, 
            173, 190, 209, 230, 253, 279, 307, 337, 371, 408, 449, 494, 544, 598, 658, 
            724, 796, 876, 963, 1060, 1166, 1282, 1411, 1552, 1707, 1878, 2066, 2272, 2499, 
            2749, 3024, 3327, 3660, 4026, 4428, 4871, 5358, 5894, 6484, 7132, 7845, 8630, 
            9493, 10442, 11487, 12635, 13899, 15289, 16818, 18500, 20350, 22385, 24623, 
            27086, 29794, 32767};


        public ImaAdpcmDecoder()
        {
            // Set initial state
            Reset();
        }

        /// <summary>
        /// IMA-ADPCM Decoder
        /// </summary>
        public short[] Decode(byte[] ibuf)
        {
            // ibuf[i]      quantized residue            
            int  diff; // de-quantized residue
            
            int  step; // current quantizer step
            int  i;
            byte code;

            short[] obuf = new short[ibuf.Length];


            for (i = 0; i < ibuf.Length; i++)
            {
                code = (byte) (ibuf[i] & 0x0F);
                
                // Reconstruct the residue
                step = _deltaTable[_index];
                diff = 0;

                if ((code & 4) != 0) diff += step;                
                if ((code & 2) != 0) diff += step >> 1;
                if ((code & 1) != 0) diff += step >> 2;
                if ((code & 8) != 0) diff = -diff;

                // Genrerate de predicted sample using the reconstructed residue
                _pred += diff;
                if (_pred > +32767) _pred = +32767;
                if (_pred < -32768) _pred = -32768;

                obuf[i] = (short) _pred;

                // Update the internal state (_index variable)
                _index += _indexTable[code];
                if (_index < 00) _index = 00;
                if (_index > 88) _index = 88;
            }

            return obuf;
        }



        /// <summary>
        /// Reset internal state to zero (predicted sample acumulator and index register)
        /// </summary>
        public void Reset()
        {
            _index = 0;
            _pred  = 0;
        }

    }
}
